#https://www.bioconductor.org/packages/release/bioc/vignettes/bsseq/inst/doc/bsseq.html
#module load R/4.0.0-foss-2020a
#R
args = commandArgs(trailingOnly=TRUE)

library(bsseq)

load(file=paste("BS4samples.chr",args[1],".rda", sep=""))

#check data
head(granges(BS4samples.chr22))
head(getCoverage(BS4samples.chr22 , type='M'))
head(getCoverage(BS4samples.chr22 , type='Cov'))
head(start(BS4samples.chr22))
head(seqnames(BS4samples.chr22))
sampleNames(BS4samples.chr22)
pData(BS4samples.chr22)
dim(BS4samples.chr22)
head(getMeth(BS4samples.chr22,type="raw"))


BS4samples.chr22.fit<-BSmooth(BS4samples.chr22)
BS4samples.chr22.cov <- getCoverage(BS4samples.chr22.fit)
#keep rows with at least one column has >=4 coverage
keepLoci.ex <- which(rowSums(BS4samples.chr22.cov[, c("imr90_r1", "imr90_r2")] >= 4) >= 2 &
                     rowSums(BS4samples.chr22.cov[, c("h1_r1", "h1_r2")] >= 4) >= 2)
BS4samples.chr22.tstat <- BSmooth.tstat(BS4samples.chr22.fit[keepLoci.ex,], 
                                    group1 = c("imr90_r1", "imr90_r2"),
                                    group2 = c("h1_r1", "h1_r2"), 
                                    estimate.var = "group2",
                                    local.correct = TRUE,
                                    verbose = TRUE)

dmrs0 <- dmrFinder(BS4samples.chr22.tstat, cutoff = c(-4.6, 4.6), verbose = TRUE)
#default maxGap is 300bp, at least 3 CpG in one DMR
dmrs <- subset(dmrs0, abs(meanDiff) > 0.1 & n >= 3)

chrs= args[1]
write.table(dmrs[, c('chr', 'start','end')], 
            sep="\t", row.names=FALSE, col.names=FALSE, 
            quote=FALSE, file=paste(chrs,"_BSmooth_dmrs_cov4gt2.txt", sep=""))


