#https://bioconductor.org/packages/devel/bioc/vignettes/methylKit/inst/doc/methylKit.html#35_Tiling_windows_analysis
library(methylKit)

args = commandArgs(trailingOnly = TRUE)


file.list = list(paste(args[1], "/mc.imr90.r1.",args[5],"_minCov4",".bed", sep=""),
                paste(args[2],  "/mc.imr90.r2.",args[5],"_minCov4",".bed", sep=""),
                paste(args[3],  "/mc.h1.r1.",args[5],"_minCov4",".bed",sep=""),
                paste(args[4],  "/mc.h1.r2.",args[5],"_minCov4",".bed",sep="") )

myobj = methRead(file.list,
                 pipeline=list(fraction=TRUE,chr.col=1,start.col=2,end.col=3,coverage.col=5,freqC.col=4,strand.col=6),
                 header=FALSE,
                 sample.id=list("imr90_r1", "imr90_r2","h1_r1","h1_r2"),
                 assembly="hg18",
                 treatment=c(1,1,0, 0),
                 context="CpG" )

meth=unite(myobj)

#CpG analysis
#myDiff=calculateDiffMeth(meth)
#myDiff25p=getMethylDiff(myDiff)
#out=getData(myDiff25p)

#tiled 200bp time=1.5 minutes , real 79.92 user 79.43
#tiled 25bp time=2 minutes, real real 102.14, user 101.00

#tiled window analysis

tiles = tileMethylCounts(meth,win.size=as.numeric(args[6]),step.size=as.numeric(args[7]))
tiled_myDiff=calculateDiffMeth(tiles)
tiled_myDiff25p=getMethylDiff(tiled_myDiff)
tiled_out=getData(tiled_myDiff25p)
chrs= args[5]
write.table(tiled_out[, c('chr', 'start','end')], sep="\t", row.names=FALSE, col.names=FALSE, quote=FALSE, file=paste(chrs, "_methylKit_tl",args[6],"b_dmrs.txt", sep=""))




