import sys
import tarfile
import subprocess
import urllib, requests
import time
import os
import shutil
import glob
from mc2bed_and_filter import mc2bed
from get_HMST_dmrs4all import write_HMST_dmrs_toFile
from overlap_count_table import overlap_c_table
def comparison4packages(input_data = 'in_data/human/WGBS-data/nature08514/',
                       output = 'data/out/',
                       chromosome = 22 ,
                       data_availabe = True,
                       win_size = [200, 25]):

    chr = str(chromosome)
    output_path = output + chr + '/'
    if not os.path.exists(output_path):
        os.makedirs(output_path, exist_ok=True)
    if not os.path.exists(input_data):
        os.makedirs(input_data, exist_ok= True)
    empty_dir(output_path)

    if data_availabe:
        print("Check if data is really avaiblable or not. ")

    else:
        imr90_r1_url = 'http://neomorph.salk.edu/ftpdata/ftpuser3/mc/mc_imr90_r1.tar.gz'
        imr90_r2_url = 'http://neomorph.salk.edu/ftpdata/ftpuser3/mc/mc_imr90_r2.tar.gz'
        h1_r1_url = 'http://neomorph.salk.edu/ftpdata/ftpuser3/mc/mc_h1_r1.tar.gz'
        h1_r2_url = 'http://neomorph.salk.edu/ftpdata/ftpuser3/mc/mc_h1_r2.tar.gz'

        for url in [imr90_r1_url, imr90_r2_url, h1_r1_url, h1_r2_url]:
            print("Now downloading .. ", url)
            urllib.request.urlretrieve(url, os.path.join(input_data, url.split('/')[-1]))
            tar = tarfile.open(os.path.join(input_data, url.split('/')[-1]))
            tar.extractall(input_data)
            tar.close()
    print("Running step 0. ")

    r_command = 'Rscript'
    sh_command = 'sh'
    get_BS_path = 'get_BS4wang.chr22.R'
    imr90_r1_parameter = os.path.join(input_data, 'mc_imr90_r1')
    imr90_r2_parameter = os.path.join(input_data, 'mc_imr90_r2')
    h1_r1_parameter = os.path.join(input_data, 'mc_h1_r1')
    h1_r2_parameter = os.path.join(input_data, 'mc_h1_r2')

    cmd = [r_command, '--vanilla' , get_BS_path, os.path.join(imr90_r1_parameter + '/mc_imr90_r1_' + chr),
                                                os.path.join(imr90_r2_parameter + '/mc_imr90_r2_' + chr),
                                                os.path.join(h1_r1_parameter + '/mc_h1_r1_' + chr),
                                                os.path.join(h1_r2_parameter + '/mc_h1_r2_' + chr),
                                                chr]

    subprocess.call(cmd)







    print("Running step 1. ")

    files = [os.path.join(imr90_r1_parameter + '/mc_imr90_r1_' + chr),
            os.path.join(imr90_r2_parameter + '/mc_imr90_r2_' + chr),
            os.path.join(h1_r1_parameter + '/mc_h1_r1_' + chr),
            os.path.join(h1_r2_parameter + '/mc_h1_r2_' + chr)]

    for file in files:
        mc2bed(file)


    with open('times.txt', 'w') as f:


        print("Running step 2. Analyzing packages of R ")
        analyz_bs = 'analyze_BS.R'
        analyz_mk = 'analyze_methylkit.R'
        analyz_ms = 'analyze_mlSg.R'

        print("========analyzing BS============")

        start = time.time()

        subprocess.call([r_command, "--vanilla", analyz_bs, chr])


        f.writelines("Time for BS: " + str(time.time()-start)+"\n")
        print("----------------------Time for BS: " + str(time.time()-start)+"---------------------\n")




        for ws in win_size:
            print("========analyzing methylkit for ",ws," window size.")

            start = time.time()
            subprocess.call([r_command, "--vanilla", analyz_mk,
                         imr90_r1_parameter,
                         imr90_r2_parameter,
                         h1_r1_parameter,
                         h1_r2_parameter,
                         chr,
                         str(ws),
                         str(ws)])
            f.writelines("Time for methylKit " + str(ws) +" window size " + str(time.time()- start)+"\n")
            print("----------------------Time for methylKit " + str(ws) + " window size " + str(time.time()-start) + "---------------------\n")

            print("========analyzing methylsig for ",ws," window size.")

            start = time.time()

            subprocess.call([r_command, "--vanilla", analyz_ms,
                             chr,
                             str(ws)])

            f.writelines("Time for methylSig " + str(ws) + " window size " + str(time.time()-start)+"\n")
            print("----------------------Time for methylSig " + str(ws) + " window size " + str(time.time()-start) + "---------------------\n")

        print("Running step 3. Analyzing package HMST ")


        run_wgbs = './run_WGBS.sh'
        subprocess.call([sh_command,run_wgbs,
                         input_data,
                         chr])
        start = time.time()
        run_wgbs = './run_WGBS_t_ga.sh'
        subprocess.call([sh_command, run_wgbs,
                         input_data,
                         chr])
        f.writelines("Time for HMST Gene Annotation " + str(time.time() - start) + "\n")
        print("----------------------Time for HMST Gene Annotation " + str(time.time() - start) + "---------------------\n")

        start = time.time()
        run_wgbs = './run_WGBS_t_DMR.sh'
        subprocess.call([sh_command, run_wgbs,
                         input_data,
                         chr])
        f.writelines("Time for HMST DMR search " + str(time.time()-start)+"\n")
        print("----------------------Time for HMST DMR Search " + str(time.time()-start)+"---------------------\n")

        run_wgbs = './run_WGBS_at.sh'
        subprocess.call([sh_command, run_wgbs,
                         input_data,
                         chr])

        print("Running step 4. Obtain all results from HMST ")


        file_folder = chr + '_WGBS/data/'
        # file_folder='out_hmst_isST1_ttest_chr22/'
        files = [file_folder + '/*TES*DMRs_all.csv', file_folder + '/*TSS*DMRs_all.csv',
                 file_folder + '/*geneBody*DMRs_all.csv', file_folder + '/*intergenic*DMRs_all.csv',
                 file_folder + '/*5dist*DMRs_all.csv'
                 ]

        new = []
        for f in files:
            if '*' in f:
                tmp_f = glob.glob(f)
                new = new + tmp_f
            else:
                new.append(f)
        # print(new)
        write_HMST_dmrs_toFile(new, chr, chr + '_HMST_dmrs')



        print("Running step 5. Compare HMST vs R packages")

        subprocess.call([sh_command,'./sort_files', chr])
        subprocess.call([sh_command,'./merge_files', chr])
        subprocess.call([sh_command,'./count_overlap_in_file', chr])

        print("Running step 6. Final Table")

        f_bsseq = ['chr'+chr+'_BSmooth_dmrs_cov4gt2_sorted_merged.txt',
                   'chr'+chr+'_methylSig_tl200b_dmrs_cov4gt2_sorted_merged.txt',
                   'chr'+chr+'_methylKit_tl200b_dmrs_sorted_merged.txt',
                   'chr'+chr+'_methylSig_tl25b_dmrs_cov4gt2_sorted_merged.txt',
                   'chr'+chr+'_methylKit_tl25b_dmrs_sorted_merged.txt']


        f_hmst_vs_bsseq = ['chr'+chr+'_HMST_dmrs_common_vs_BSmooth_dmrs.csv',
                           'chr'+chr+'_HMST_dmrs_common_vs_methylsig_tl200_dmrs.csv',
                           'chr'+chr+'_HMST_dmrs_common_vs_methylkit_tl200_dmrs.csv',
                           'chr'+chr+'_HMST_dmrs_common_vs_methylsig_tl25_dmrs.csv',
                           'chr'+chr+'_HMST_dmrs_common_vs_methylkit_tl25_dmrs.csv']

        overlap_c_table(chr, chr+'_HMST_dmrs_common_sorted_merged.csv', f_bsseq, f_hmst_vs_bsseq)

        for i in glob.glob('*'+chr+'*.txt'):
            shutil.move(i, output_path)
        for i in glob.glob('*'+chr+'*.csv'):
            shutil.move(os.path.join(os.getcwd(),i), os.path.join(output_path,i))
        shutil.move(chr+'_WGBS', output_path)

def empty_dir(folder):
    # function for deleting files from a folder

    for filename in os.listdir(folder):
        file_path = os.path.join(folder, filename)
        try:
            if os.path.isfile(file_path) or os.path.islink(file_path):
                os.unlink(file_path)
            elif os.path.isdir(file_path):
                shutil.rmtree(file_path)
        except Exception as e:
            print(('Failed to delete %s. Reason: %s' % (file_path, e)))

if __name__ == "__main__":
    if len(sys.argv) < 2:
        print("Usage: python comparison.py <chromosome number>")
        exit()

    try:
        chromosome = sys.argv[1]
        in_data = sys.argv[2]
        out_data = sys.argv[3]
        data_available = sys.argv[4]
    except:
        in_data = 'in_data/human/WGBS-data/nature08514/'
        out_data = 'data/out/'
        data_available = True

    window_size = [200, 25]
    print(chromosome)
    comparison4packages(in_data, out_data , chromosome , data_available, window_size)
