#! /usr/bin/Rscript

## The following script downloads and constructs the BS.chr22 dataset,
#1. dowload full data
#wget http://neomorph.salk.edu/ftpdata/ftpuser3/mc/mc_imr90_r2.tar.gz
#wget http://neomorph.salk.edu/ftpdata/ftpuser3/mc/mc_h1_r1.tar.gz
#wget http://neomorph.salk.edu/ftpdata/ftpuser3/mc/mc_imr90_r1.tar.gz
#wget http://neomorph.salk.edu/ftpdata/ftpuser3/mc/mc_h1_r2.tar.gz

## included in the bsseq package
#module load R/4.0.0-foss-2020a

library(bsseq)

## First we download.  Each file is slightly less than 200 MB
#download.file(
#    url = "http://neomorph.salk.edu/ftpdata/ftpuser3/mc/mc_imr90_r1.tar.gz",
#    destfile = "mc_imr90_r1.tar.gz")

#download.file(
#    url = "http://neomorph.salk.edu/ftpdata/ftpuser3/mc/mc_imr90_r2.tar.gz",
#    destfile = "mc_imr90_r2.tar.gz")

#2. after downloading full data, extract chromsome data by using R:
#module load R/4.0.0-foss-2020a
#library(bsseq)
#untar("mc_imr90_r1.tar.gz",  "mc_imr90_r1/mc_imr90_r1_22", compressed = TRUE)
#untar("mc_imr90_r2.tar.gz",  "mc_imr90_r2/mc_imr90_r2_22", compressed = TRUE)
#untar("mc_h1_r1.tar.gz",  "mc_h1_r1/mc_h1_r1_22", compressed = TRUE)
#untar("mc_h1_r2.tar.gz",  "mc_h1_r2/mc_h1_r2_22", compressed = TRUE)


#3. an example of extracted chr22 data is in: 
#in_data/human/WGBS-data/nature08514/

## Now the workhorse function
#4. generate binary R data for the further data analysis by using below scripts
#

read.lister <- function(file) {
    dat <- read.table(
        file,
        skip = 1,
        row.names = NULL,
        col.names = c("chr", "pos", "strand", "context", "M", "Cov"),
        colClasses = c("character", "integer", "character", "character",
                       "integer", "integer"))
    ## we remove all non-CpG calls.  This includes SNPs
    dat <- dat[dat$context == "CG", ]
    dat$context <- NULL
    dat$chr <- paste("chr", dat$chr, sep = "")
    ## Now we need to handle that the data has separate lines for each strand
    ## We join these
    tmp <- dat[dat$strand == "+",]
    BS.forward <- BSseq(
        pos = tmp$pos,
        chr = tmp$chr,
        M = as.matrix(tmp$M, ncol = 1),
        Cov = as.matrix(tmp$Cov, ncol = 1),
        sampleNames = "forward")
    tmp <- dat[dat$strand == "-",]
    BS.reverse <- BSseq(
        pos = tmp$pos - 1L,
        chr = tmp$chr,
        M = as.matrix(tmp$M, ncol = 1),
        Cov = as.matrix(tmp$Cov, ncol = 1),
        sampleNames = "reverse")
    BS <- combine(BS.forward, BS.reverse)
    BS <- collapseBSseq(BS, group = c("a", "a"), type = "integer")
    BS
}

# BS.imr90_r1 <- read.lister("in_data/human/WGBS-data/nature08514/mc_imr90_r1/mc_imr90_r1_22")
# sampleNames(BS.imr90_r1) <- "imr90_r1"
# BS.imr90_r2 <- read.lister("in_data/human/WGBS-data/nature08514/mc_imr90_r2/mc_imr90_r2_22")
# sampleNames(BS.imr90_r2) <- "imr90_r2"
#
# BS.h1_r1 <- read.lister("in_data/human/WGBS-data/nature08514/mc_h1_r1/mc_h1_r1_22")
# sampleNames(BS.h1_r1) <- "h1_r1"
# BS.h1_r2 <- read.lister("in_data/human/WGBS-data/nature08514/mc_h1_r2/mc_h1_r2_22")
# sampleNames(BS.h1_r2) <- "h1_r2"
#
#
# BS4samples.chr22 <- combine(BS.imr90_r1,BS.imr90_r2 ,BS.h1_r1,BS.h1_r2)
#
# pData(BS4samples.chr22)$Rep <- c("imr90_replicate1", "imr90_replicate2","h1_replicate1", "h1_replicate2")
# validObject(BS4samples.chr22)
# pData(BS4samples.chr22)
#
# save(BS4samples.chr22, file = "BS4samples.chr22.rda")
# #tools::resaveRdaFiles("BS4samples.chr22.rda")

args = commandArgs(trailingOnly=TRUE)


BS.imr90_r1 <- read.lister(args[1])
sampleNames(BS.imr90_r1) <- "imr90_r1"
BS.imr90_r2 <- read.lister(args[2])
sampleNames(BS.imr90_r2) <- "imr90_r2"

BS.h1_r1 <- read.lister(args[3])
sampleNames(BS.h1_r1) <- "h1_r1"
BS.h1_r2 <- read.lister(args[4])
sampleNames(BS.h1_r2) <- "h1_r2"


BS4samples.chr22 <- combine(BS.imr90_r1,BS.imr90_r2 ,BS.h1_r1,BS.h1_r2)


pData(BS4samples.chr22)$Rep <- c("imr90_replicate1", "imr90_replicate2","h1_replicate1", "h1_replicate2")
validObject(BS4samples.chr22)
pData(BS4samples.chr22)

save(BS4samples.chr22, file = paste("BS4samples.chr",args[5],".rda", sep= ""))
#tools::resaveRdaFiles("BS4samples.chr22.rda")