#!/Users/sendre/anaconda3/bin/python
import sys
#sys.path.insert(0, 'scripts_low')
#sys.path.insert(0, 'scripts_high')
#sys.path.insert(0, 'scripts_high.scripts')
import argparse
import warnings
warnings.filterwarnings("ignore")


class Main(object):
    def __init__(self):
        parser = argparse.ArgumentParser(
            description='HMST-Seq Analyzer',
            usage='''hmst_seq_analyzer <task> [<args>]

Tasks available for using:
   gene_annotation            Cleans reference file and creates genomic region files
                              (TSS, geneBody, TES, 5dist and intergenic) from the reference

   data_preprocessing         Creation of 5mC and 5hmC files, quantile normalization

   find_MRs                   Extracts genomic regions from 5mC/5hmC-files and finds
                              methylated regions

   prepare_for_DMR_finding    Finds overlapping methylated regions between MRs in
                              WT condition samples and KO condition samples

   DMR_search                 Finds differentially methylated regions

   prep4plot                  Prepares files for plotting

   plot_all                   Plots hyper versus hypo differentially methylated regions,
                              enhancer methylated regions, TSS_gene_TES methylated regions
                              and relative density of significantly modified sites in MRs
                              with versus all sites in MRs

   clean_files                Removes some unwanted files. Please only use after prep4plot
                              is already done
''')
        parser.add_argument('task', help='Pipeline task to run')
        # parse_args defaults to [1:] for args, but you need to
        # exclude the rest of the args too, or validation will fail
        args = parser.parse_args(sys.argv[1:2])
        if not hasattr(self, args.task):
            print('***** ERROR: Unrecognized task *****')
            parser.print_help()
            exit(1)
        # use dispatch pattern to invoke method with same name
        getattr(self, args.task)()

    def gene_annotation(self):
        from .scripts_high.gene_annotation import my_parser, run
        parser = my_parser(argparse.ArgumentParser(prog='hmst_seq_analyzer gene_annotation',
            description='Cleans reference file and creates genomic region files (TSS, geneBody, TES, 5dist and intergenic) from the reference'))

        run(parser.parse_args(sys.argv[2:]))

    def data_preprocessing(self):
        from .scripts_high.data_preprocessing import my_parser, run
        parser = my_parser(argparse.ArgumentParser(prog='hmst_seq_analyzer data_preprocessing',
            description='Creation of 5mC and 5hmC files, quantile normalization'))

        run(parser.parse_args(sys.argv[2:]))

    def find_MRs(self):
        from .scripts_high.find_MRs import my_parser, run
        parser = my_parser(argparse.ArgumentParser(prog='hmst_seq_analyzer find_MRs',
            description='Extracts genomic regions from 5mC/5hmC-files and finds methylated regions'))

        run(parser.parse_args(sys.argv[2:]))

    def prepare_for_DMR_finding(self):
        from .scripts_high.prepare_for_DMR_finding import my_parser, run
        parser = my_parser(argparse.ArgumentParser(prog='hmst_seq_analyzer prepare_for_DMR_finding',
            description='Finds overlapping methylated regions between MRs in WT condition samples and KO condition samples'))

        run(parser.parse_args(sys.argv[2:]))

    def DMR_search(self):
        from .scripts_high.DMR_search import my_parser, run
        parser = my_parser(argparse.ArgumentParser(prog='hmst_seq_analyzer DMR_search',
            description='Finds differentially methylated regions'))

        run(parser.parse_args(sys.argv[2:]))

    def prep4plot(self):
        from .scripts_high.prep4plot import my_parser, run
        parser = my_parser(argparse.ArgumentParser(prog='hmst_seq_analyzer prep4plot',
            description='Prepares files for plotting'))

        run(parser.parse_args(sys.argv[2:]))

    def plot_all(self):
        from .scripts_high.plot_all import my_parser, run
        parser = my_parser(argparse.ArgumentParser(prog='hmst_seq_analyzer plot_all',
            description='Plots hyper versus hypo differentially methylated regions, enhancer methylated regions, TSS_gene_TES methylated regions and relative density of significantly modified sites in MRs with versus all sites in MRs'))

        run(parser.parse_args(sys.argv[2:]))

    def clean_files(self):
        from .scripts_high.remove_filtered_formatted import my_parser, run
        parser = my_parser(argparse.ArgumentParser(prog='hmst_seq_analyzer clean_files',
            description='Removes some unwanted files. Please only use after prep4plot is already done'))

        run(parser.parse_args(sys.argv[2:]))

def main():
    Main()

if __name__ == '__main__':
    Main()
