import argparse
import os
import sys
from .scripts.plot_DMRs_percentage import main as plt_dmr_main
from .scripts.plot_enhancer import main as plt_enh_main
from .scripts.plot_TSSgeneTES import main as plt_tgt_main
from .scripts.plot_regions import main as plt_reg_main

def plt_TSS_geneBody_TES_enhancer(MR_file, MRs_txt, plot_TGT, plot_ENH, tissue, enhX, enhY, w, window, sigma, gX, gY, G, folder_out):
    t_g_t_e = [line.rstrip('\n') for line in open(MR_file)]
    enhancers_mc = []; enhancers_hmc = []
    tss_mc = []; tss_hmc = []
    tes_mc = []; tes_hmc = []
    gbody_mc = []; gbody_hmc = []
    for file in t_g_t_e:
        meth = file.split('/')[-1].split('_')[1]
        region = file.split('/')[-1].split('_')[2]
        if region == 'enhancer':
            if meth == '5mC':
                enhancers_mc.append(file)
            elif meth == '5hmC':
                enhancers_hmc.append(file)
        elif region == 'TES':
            if meth == '5mC':
                tes_mc.append(file)
            elif meth == '5hmC':
                tes_hmc.append(file)
        elif region == 'TSS':
            if meth == '5mC':
                tss_mc.append(file)
            elif meth == '5hmC':
                tss_hmc.append(file)
        elif region == 'geneBody':
            if meth == '5mC':
                gbody_mc.append(file)
            elif meth == '5hmC':
                gbody_hmc.append(file)

    if plot_ENH in ('yes', 'y'):
        if len(enhancers_mc) > 0:
            plt_enh_main(enhancers_mc, tissue, enhX, enhY, w, window, sigma, MRs_txt, folder_out)
        if len(enhancers_hmc) > 0:
            plt_enh_main(enhancers_hmc, tissue, enhX, enhY, w, window, sigma, MRs_txt, folder_out)
    elif plot_ENH in ('no', 'n'):
        pass
    else:
        print('***** WARNING: plot_ENH value is not known, must be in [yes, y, no, n]. Continuing as if input is no. *****')

    if plot_TGT in ('yes', 'y'):
        if len(tss_mc)+len(tes_mc)+len(gbody_mc) > 0:
            plt_tgt_main(tss_mc, tes_mc, gbody_mc, gX, gY, G, w, window, sigma, MRs_txt, folder_out)
        if len(tss_hmc)+len(tes_hmc)+len(gbody_hmc) > 0:
            plt_tgt_main(tss_hmc, tes_hmc, gbody_hmc, gX, gY, G, w, window, sigma, MRs_txt, folder_out)
    elif plot_TGT in ('no', 'n'):
        pass
    else:
        print('***** WARNING: plot_TGT value is not known, must be in [yes, y, no, n]. Continuing as if input is no. *****')

def main(list_count_regions, list_count_sites, counts_DMRs_mc, counts_DMRs_hmc,
         list_allMRs, list_overlappingMRs, plot_TGT, plot_ENH, gX, gY, G, enhX, enhY, w, window, sigma,
         tissue, folder_out):
    """Calls the respective plot scripts with the respective input files. Users
    can input one or more files as arguments - only the plots created from the files
    that are input will be plotted"""

    folder_out += '/plots'

    if counts_DMRs_mc is None and counts_DMRs_hmc is None:
        pass
    elif counts_DMRs_mc is None or counts_DMRs_hmc is None:
        print('***** Error in plot_all.py: both 5mC and 5hmC DMR count files must be provided if hyper vs hypo DMRs should be plotted. continuing. *****')
        pass
    else:
        #counts_DMRs_mc, counts_DMRs_hmc -> plot_DMRs_percentage
        try:
            plt_dmr_main(counts_DMRs_mc, counts_DMRs_hmc, folder_out)
        except IOError:
            print('IOError: '+counts_DMRs_mc+' OR '+counts_DMRs_hmc+'. continuing.')


    #list_count_regions, list_count_sites -> plot_regions
    count_regs = []
    count_sites = []
    if list_count_regions is not None:
        try:
            count_regs = [line.rstrip('\n') for line in open(list_count_regions)]
        except IOError:
            print('IOError: '+list_count_regions+'. continuing.')
            list_count_regions = None

    if list_count_sites is not None:
        try:
            count_sites = [line.rstrip('\n') for line in open(list_count_sites)]
        except IOError:
            print('IOError: '+list_count_sites+'. continuing.')
            list_count_sites = None

    if list_count_regions is not None and list_count_sites is not None:
        plt_reg_main(count_regs, count_sites, folder_out)

    if list_allMRs is not None:
        try:
            plt_TSS_geneBody_TES_enhancer(list_allMRs, 'allMRs', plot_TGT, plot_ENH,
                        tissue, enhX, enhY, w, window, sigma, gX, gY, G, folder_out)
        except IOError:
            print('IOError: '+list_allMRs+'. continuing.')

    if list_overlappingMRs is not None:
        try:
            plt_TSS_geneBody_TES_enhancer(list_overlappingMRs, 'overlappingMRs',
                        plot_TGT, plot_ENH, tissue, enhX, enhY, w, window, sigma,
                        gX, gY, G, folder_out)
        except IOError:
            print('IOError: '+list_overlappingMRs+'. continuing.')

def my_parser(parser):
    in_files = parser.add_argument_group("Input files")
    in_files.add_argument("-reg", "--listCountRegions",
                        metavar='file',
                        help="the list file containing region counts: list_count_allMRs_regions_files.txt. For plotting relative density")
    in_files.add_argument("-sit", "--listCountSites",
                        metavar='file',
                        help="the list file containing site counts: list_count_sites_files.txt. For plotting relative density")
    in_files.add_argument("-cmc", "--countsDMRsMC",
                        metavar='file',
                        help="the DMR 5mC count file, output from the DMR_search task")
    in_files.add_argument("-chmc", "--countsDMRsHMC",
                        metavar='file',
                        help="the DMR 5hmC count file, output from the DMR_search task")
    in_files.add_argument("-aMR", "--allMRs",
                        metavar='file',
                        help="The file: list_TSS_genebody_TES_enhancer_allMRs.txt, output from the prep4plot task")
    in_files.add_argument("-oMR", "--overlappingMRs",
                        metavar='file',
                        help="The file: list_overlapping_MRs.txt, output from the prepare_for_DMR_finding task if there is only one KO and one WT sample")


    tssgenetes = parser.add_argument_group("Options for TSS_gene_TES plot")
    tssgenetes.add_argument("--plotTGT",
                        default='yes',
                        metavar='',
                        help="if listPlotFile is provided, plot TSS_gene_TES plot? [yes/y] or [no/n]. default=yes")
    tssgenetes.add_argument("-gX",
                        type=int,
                        default=1000,
                        metavar='',
                        help="the number of upstream bp, TSS, TES, gene. default=1000")
    tssgenetes.add_argument("-gY",
                        type=int,
                        default=1000,
                        metavar='',
                        help="the number of downstream bp, TSS, TES, gene. default=1000")
    tssgenetes.add_argument("-G",
                        type=int,
                        default=4000,
                        metavar='',
                        help="the size/number of window bins geneBody will be divided into. default=4000")

    enhancer = parser.add_argument_group("Options for enhancer plot")
    enhancer.add_argument("--plotENH",
                        default='yes',
                        metavar='',
                        help="if --allMRs and/or --overlappingMRs is provided and enhancers are available, plot enhancer plot? [yes/y] or [no/n]. default=yes")
    enhancer.add_argument("-enhX",
                        type=int,
                        default=5000,
                        metavar='',
                        help="the number of upstream bp from center in enhancer region. default=5000")
    enhancer.add_argument("-enhY",
                        type=int,
                        default=5000,
                        metavar='',
                        help="the number of downstream bp from center in enhancer region. default=5000")
    enhancer.add_argument("-t", "--tissue",
                        default='all',
                        metavar='',
                        help="name of an enhancer tissue, to plot just this specific tissue. default=all")

    optional = parser.add_argument_group("Optional, has default values")
    optional.add_argument("-ws",
                        type=int,
                        default=500,
                        metavar='',
                        help="window size for rolling window smoothing, for plotting of TSS_geneBody_TES and enhancer. default=500")
    optional.add_argument("-w",
                        type=int,
                        default=100,
                        metavar='',
                        help="step size for interpolation, for plotting of TSS_gene_TES and enhancer. default=100")
    optional.add_argument("-s", "--sigma",
                        type=int,
                        default=50,
                        metavar='',
                        help="sigma, used for smoothing the plots. the higher the number, the smoother the plots. default=50")
    optional.add_argument("-F", "--folderOut",
                        default='out',
                        metavar='',
                        help="what should be the name of the out-folder. default=out")
    return parser
def run(args):
    """Calls the main function with the unpacked arguments"""
    if args.folderOut[-1] == '/':
        args.folderOut = args.folderOut[:-1]

    if not os.path.exists(args.folderOut):
        os.system('mkdir '+args.folderOut)

    if not os.path.exists(args.folderOut+'/plots'):
        os.system('mkdir '+args.folderOut+'/plots')

    if not os.path.exists(args.folderOut+'/plots/plotData'):
        os.system('mkdir '+args.folderOut+'/plots/plotData')

    main(args.listCountRegions, args.listCountSites, args.countsDMRsMC,
         args.countsDMRsHMC, args.allMRs, args.overlappingMRs, args.plotTGT, args.plotENH, args.gX,
         args.gY, args.G, args.enhX, args.enhY, args.w, args.ws, args.sigma, args.tissue,
         args.folderOut)


if __name__ == "__main__":
    args = my_parser(argparse.ArgumentParser('python prepare_for_DMR_finding.py')).parse_args()

    run(args)
