import argparse
import os
import sys
from .scripts.prepare_MRs_for_plotting import main as prep_4pl_main
from .scripts.compare_5mC_5hmC_MRs_regions import main as prep_reg_main

def main(ko_mr_files, wt_mr_files, cutoff, folder_out):
    """Calls prepare_MRs_for_plotting and compare_5mC_5hmC_MRs_regions, """
    mr_kos = [line.rstrip('\n') for line in open(ko_mr_files)]
    mr_wts = [line.rstrip('\n') for line in open(wt_mr_files)]


    # Create MR files for TSS, gene, TES and enhancer
    regions_in_filename = []
    prep_for_plot_files = []
    for file in mr_kos+mr_wts:
        if 'tss_up' in file.lower():
            prep_for_plot_files.append(file)
            if 'TSS' not in regions_in_filename:
                regions_in_filename.append('TSS')
        elif 'tes_up' in file.lower():
            prep_for_plot_files.append(file)
            if 'TES' not in regions_in_filename:
                regions_in_filename.append('TES')
        elif 'gene_up' in file.lower():
            prep_for_plot_files.append(file)
            if 'geneBody' not in regions_in_filename:
                regions_in_filename.append('geneBody')
        elif 'enhance' in file.lower():
            prep_for_plot_files.append(file)
            if 'enhancer' not in regions_in_filename:
                regions_in_filename.append('enhancer')

    output_file = folder_out+'/list_TSS_genebody_TES_enhancer_allMRs.txt'

    region_files_list = folder_out+'/list_count_allMRs_regions_files.txt'

    folder_out += '/data'
    MR_files = prep_4pl_main(prep_for_plot_files, folder_out)

    out_f = open(output_file, 'w')
    for f in MR_files:
        out_f.write(f+'\n')
    out_f.close()

    # Create 5mC 5hmC comparison regions for each sample
    sample_dict = {}
    for file in mr_kos+mr_wts:
        samplename = file.split('/')[-1].split('_')[0]
        if samplename not in sample_dict:
            sample_dict[samplename] = {}

        if 'tss_up' in file.lower():
            if 'TSS' in sample_dict[samplename]:
                sample_dict[samplename]['TSS'].append(file)
            else:
                sample_dict[samplename]['TSS'] = [file]
        elif 'tes_up' in file.lower():
            if 'TES' in sample_dict[samplename]:
                sample_dict[samplename]['TES'].append(file)
            else:
                sample_dict[samplename]['TES'] = [file]
        elif 'gene_up' in file.lower():
            if 'geneBody' in sample_dict[samplename]:
                sample_dict[samplename]['geneBody'].append(file)
            else:
                sample_dict[samplename]['geneBody'] = [file]
        elif '5dist_up' in file.lower():
            if '5dist' in sample_dict[samplename]:
                sample_dict[samplename]['5dist'].append(file)
            else:
                sample_dict[samplename]['5dist'] = [file]
        elif 'intergenic' in file.lower():
            if 'intergenic' in sample_dict[samplename]:
                sample_dict[samplename]['intergenic'].append(file)
            else:
                sample_dict[samplename]['intergenic'] = [file]
        elif 'enhance' in file.lower():
            if 'enhancer' in sample_dict[samplename]:
                sample_dict[samplename]['enhancer'].append(file)
            else:
                sample_dict[samplename]['enhancer'] = [file]

    region_f_list = open(region_files_list, 'w')
    for key, values in sample_dict.items():
        all_regions = list(values.keys())
        if 'TSS' not in all_regions:
            values['TSS'] = None
        if 'TES' not in all_regions:
            values['TES'] = None
        if 'geneBody' not in all_regions:
            values['geneBody'] = None
        if '5dist' not in all_regions:
            values['5dist'] = None
        if 'intergenic' not in all_regions:
            values['intergenic'] = None
        if 'enhancer' not in all_regions:
            values['enhancer'] = None
        reg_file = prep_reg_main(values['TSS'], values['TES'], values['geneBody'],
                                 values['5dist'], values['intergenic'], values['enhancer'], cutoff, folder_out)

        region_f_list.write(reg_file+'\n')

    region_f_list.close()

def my_parser(parser):
    required = parser.add_argument_group("Required")
    required.add_argument("-ko", "--KOfilesMRs",
                        required=True,
                        metavar='file',
                        help="the list file containing all KO MRs filenames: list_all_filtered_formatted_MRs_KO.txt")
    required.add_argument("-wt", "--WTfilesMRs",
                        required=True,
                        metavar='file',
                        help="the list file containing all WT MRs filenames: list_all_filtered_formatted_MRs_WT.txt")

    optional = parser.add_argument_group("Optional, has default values")
    optional.add_argument("-c", "--cutoff",
                        type=float,
                        default=1.0,
                        metavar='',
                        help="minimum methylation level for a site to be considered methylated in the summary report. default=1")
    optional.add_argument("-F", "--folderOut",
                        default='out',
                        metavar='',
                        help="what should be the name of the out-folder. default=out")
    return parser

def run(args):
    """Calls the main function with the unpacked arguments"""
    if args.folderOut[-1] == '/':
        args.folderOut = args.folderOut[:-1]

    if not os.path.exists(args.folderOut):
        os.system('mkdir '+args.folderOut)

    if not os.path.exists(args.folderOut+'/data'):
        os.system('mkdir '+args.folderOut+'/data')

    main(args.KOfilesMRs, args.WTfilesMRs, args.cutoff, args.folderOut)


if __name__ == "__main__":
    args = my_parser(argparse.ArgumentParser('python prepare_for_DMR_finding.py')).parse_args()

    run(args)
