import argparse
import os
import pandas as pd
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import seaborn as sns
import numpy as np
import datetime

def get_data_ready4plot(df_count):
    """Gets the percentage D(h)MRs of overlapping MRs, differing hyper from hypo"""
    list_hyper = []
    list_hypo = []
    for i, row in df_count.iterrows():
        row_list = row.id.split('_')
        samplename = '_'.join(row_list[0:3])
        region = row_list[4]
        list_hyper.append((region, (float(row['Hyper_DMR'])/float(row['Total_MR']))*100, samplename))
        list_hypo.append((region, (float(row['Hypo_DMR'])/float(row['Total_MR']))*100, samplename))

    df_plot_hyper = pd.DataFrame(list_hyper, columns=['region', 'data', 'samplename'])
    df_plot_hypo = pd.DataFrame(list_hypo, columns=['region', 'data', 'samplename'])

    return df_plot_hyper, df_plot_hypo

def fix_plot_look(ax, title, ylabel, biggest, nbins):
    """Fixing plot title, labels etc."""
    ax.locator_params(nbins=nbins)
    ax.set(ylabel=ylabel)
    ax.set(xlabel='')
    ax.set(title=title)
    ax.legend(loc=2)
    ax.set_ylim([0, biggest+25])

def main(count_file_mc, count_file_hmc, folder_out):
    """Plots the DMR and DhMR ratios"""
    df_mc_count = pd.read_csv(count_file_mc, sep='\t', index_col=False, header=0)
    df_mc_count.rename(columns={'Unnamed: 0':'id'}, inplace=True)

    df_hmc_count = pd.read_csv(count_file_hmc, sep='\t', index_col=False, header=0)
    df_hmc_count.rename(columns={'Unnamed: 0':'id'}, inplace=True)

    mc_df_hyper, mc_df_hypo = get_data_ready4plot(df_mc_count)
    hmc_df_hyper, hmc_df_hypo = get_data_ready4plot(df_hmc_count)

    biggest = max(mc_df_hyper['data'].max(), mc_df_hypo['data'].max(),
                  hmc_df_hyper['data'].max(), hmc_df_hypo['data'].max())

    mc_df_hyper.to_csv(folder_out+'/plotData/plotData_DMRs_percentage_hyper_5mC.csv')
    mc_df_hypo.to_csv(folder_out+'/plotData/plotData_DMRs_percentage_hypo_5mC.csv')
    hmc_df_hyper.to_csv(folder_out+'/plotData/plotData_DMRs_percentage_hyper_5hmC.csv')
    hmc_df_hypo.to_csv(folder_out+'/plotData/plotData_DMRs_percentage_hypo_5hmC.csv')

    plt.rcParams.update({'font.size': 14,
                         'legend.fontsize': 12})
    #sns.set(font_scale=1.5)
    nbins = len(mc_df_hyper.index)
    fig, axs = plt.subplots(2, 2, figsize=(16,12))
    plt.subplots_adjust(hspace=0.3)
    if len(mc_df_hyper) > 0:
        bp1 = sns.barplot(x="region", hue="samplename", y="data", data=mc_df_hyper, ax=axs[0, 0])
        bp1.set_xticklabels(bp1.get_xticklabels(), rotation=30, ha='right')
        fix_plot_look(bp1.axes, 'Hyper DMRs', '% hyper-methylated regions', biggest, nbins)
    if len(mc_df_hypo) > 0:
        bp2 = sns.barplot(x="region", hue="samplename", y="data", data=mc_df_hypo, ax=axs[0, 1])
        bp2.set_xticklabels(bp2.get_xticklabels(), rotation=30, ha='right')
        fix_plot_look(bp2.axes, 'Hypo DMRs', '% hypo-methylated regions', biggest, nbins)
    if len(hmc_df_hyper) > 0:
        bp3 = sns.barplot(x="region", hue="samplename", y="data", data=hmc_df_hyper, ax=axs[1, 0])
        bp3.set_xticklabels(bp3.get_xticklabels(), rotation=30, ha='right')
        fix_plot_look(bp3.axes, 'Hyper DhMRs', '% hyper-methylated regions', biggest, nbins)
    if len(hmc_df_hypo) > 0:
        bp4 = sns.barplot(x="region", hue="samplename", y="data", data=hmc_df_hypo, ax=axs[1, 1])
        bp4.set_xticklabels(bp4.get_xticklabels(), rotation=30, ha='right')
        fix_plot_look(bp4.axes, 'Hypo DhMRs', '% hypo-methylated regions', biggest, nbins)

    plt.savefig(folder_out+'/percentage_region_DMR_5mC_5hmC_hypo_hyper'+str(datetime.datetime.now())[:10]+'.png',dpi=400)
