import pandas as pd
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import argparse
import os
import numpy as np
import seaborn as sns
import datetime

def fix_plot_look(ax, title, biggest):
    """Fixing som details on the plot"""
    ax.locator_params(nbins=10)
    #for i in np.arange(0, biggest+0.3, 0.1):
    #    ax.axhline(y=i, linewidth=0.25, color='k')
    ax.set(ylabel='Relative density of '+title)
    ax.set(xlabel='')
    ax.set(title=title)
    ax.set_ylim([0, biggest+0.5])
    ax.xaxis.grid(False)
    ax.yaxis.grid(False)

def main(reg_count_files, site_count_files, out_folder):
    """Plots relative density for MRs across genomic regions"""
    plot_mC = []
    plot_hmC = []

    if site_count_files is None and reg_count_files is None:
        print('***** ERROR in plot_regions.py: User must provide at least one region count file or at least one site count file. Exiting. *****')
        exit(1)

    if site_count_files is not None:
        for file in site_count_files:
            samplename = file.split('/')[-1].split('_')[0]
            df = pd.read_csv(file, sep='\t').loc['total']

            if df['all_mC_sites'] == 0:
                plot_mC.append(('genome', 0.0, samplename))
            else:
                plot_mC.append(('genome', float(df['total_mC'])/float(df['all_mC_sites']), samplename))
            if df['all_hmC_sites'] == 0:
                plot_hmC.append(('genome', 0.0, samplename))
            else:
                plot_hmC.append(('genome', float(df['total_hmC'])/float(df['all_hmC_sites']), samplename))

    if reg_count_files is not None:
        for file in reg_count_files:
            samplename = file.split('/')[-1].split('_')[0]

            df = pd.read_csv(file, sep='\t')
            total = df.loc['total']
            df.drop(['total'], inplace=True)


            mC = []
            for region, row in df.iterrows():
                if row['tot_mC_sites_MRs_all'] == 0:
                    plot_mC.append((region, 0.0, samplename))
                else:
                    plot_mC.append((region, float(row['total_mC_in_MRs_cutoff'])/float(row['tot_mC_sites_MRs_all']), samplename))
                if row['tot_hmC_sites_MRs_all'] == 0:
                    plot_hmC.append((region, 0.0, samplename))
                else:
                    plot_hmC.append((region, float(row['total_hmC_in_MRs_cutoff'])/float(row['tot_hmC_sites_MRs_all']), samplename))

    biggest = 0
    for i in plot_mC:
        if i[1] > biggest:
            biggest = i[1]
    for i in plot_hmC:
        if i[1] > biggest:
            biggest = i[1]

    df_plot_mC = pd.DataFrame(plot_mC, columns=['region', 'data', 'samplename'])
    df_plot_hmC = pd.DataFrame(plot_hmC, columns=['region', 'data', 'samplename'])

    df_plot_mC.to_csv(out_folder+'/plotData/plotData_relative_density_5mC.csv')
    df_plot_hmC.to_csv(out_folder+'/plotData/plotData_relative_density_5hmC.csv')

    #Plotting
    plt.rcParams.update({'font.size': 14,
                         'legend.fontsize': 11})
    #sns.set(font_scale=1.5)
    if len(df_plot_hmC[df_plot_hmC.data == 0.0]) != len(df_plot_hmC):
      #with sns.axes_style("white"):
        fig, axs = plt.subplots(ncols=2, figsize=(12,5))
        bp1 = sns.barplot(x="region", hue="samplename", y="data", data=df_plot_mC, ax=axs[0])
        bp1.set_xticklabels(bp1.get_xticklabels(), rotation=30, ha='right')
        bp2 = sns.barplot(x="region", hue="samplename", y="data", data=df_plot_hmC, ax=axs[1])
        bp2.set_xticklabels(bp2.get_xticklabels(), rotation=30, ha='right')
        #Fixing looks of plots
        axes1 = bp1.axes
        axes2 = bp2.axes
        fix_plot_look(axes1, '5mC', biggest)
        fix_plot_look(axes2, '5hmC', biggest)
    else: #WGBS data
        fig, axs = plt.subplots(ncols=1, figsize=(6,5))
        bp1 = sns.barplot(x="region", hue="samplename", y="data", data=df_plot_mC, ax=axs)
        bp1.set_xticklabels(bp1.get_xticklabels(), rotation=30, ha='right')
        #Fixing looks of plots
        axes1 = bp1.axes
        fix_plot_look(axes1, '5mC', biggest)

    plt.gcf().subplots_adjust(bottom=0.22)
    plt.savefig(out_folder+'/relative_density_'+str(datetime.datetime.now())[:10]+'.png',dpi=400)
