import pandas as pd
import numpy as np
import argparse
import os

def rem_methpoint(df):
    """Extracts methylation site and level from the IDs and puts them in
    separate columns of df"""
    new_df = pd.DataFrame(columns=['names', 'meth_point', 'power'])
    s = pd.DataFrame([x.split('||') for x in list(df.names)])
    w = pd.DataFrame([x.split(':') for x in list(s[3])])
    meth_point_ix = w.columns[-1]
    new_df['meth_point'] = w[meth_point_ix]
    new_df['power'] = df['power']
    #print(s[0]+'||'+s[1]+'||'+s[2]+'||'+w[0])
    #new_df['names'] = new_df.assign(names= s[0]+'||'+s[1]+'||'+s[2]+'||'+w[0])
    tmp_names=s[0]+'||'+s[1]+'||'+s[2]+'||'+w[0]
    #print(tmp_names.to_list())
    new_df['names']=tmp_names
    return new_df

def group_to_row(df, region):
    """extracts necessary info from df to be able to plot TSS, genebody, TES, and
    enhancer distribution, and change the format of MRs from being on multiple rows,
    to each MR being on one row only
    """
    df = df.reset_index(drop=True)
    new_df = []

    region_range = df.names[0].split('||')[0].split(':')[1:3]
    new_df.append(region_range[0])                    #region_start
    new_df.append(region_range[1])                    #region_end
    new_df.append(df.meth_point.astype(int).tolist()) #meth_points_all
    new_df.append(df.power.tolist())                  #powers
    if region == 'enhancer':
        new_df.append(df.names[0].split('||')[3])     #tissue
    else:
        new_df.append('.')
    return new_df


def main(files, out_folder):
    """Extracts necessary information from output of find_MRs
    file -- list of TSS, TES, geneBody, enhancer MR files, one or many"""
    out_filenames = []
    for file in files:
        f = file.split('/')[-1]
        samplename = f.split('_')[0]
        if '5mC' in file:
            methylation = '5mC'
        else:
            methylation = '5hmC'

        file_lower = file.lower()
        if 'tes_up' in file_lower:
            region = 'TES'
        elif 'tss_up' in file_lower:
            region = 'TSS'
        elif 'gene_up' in file_lower:
            region = 'geneBody'
        elif 'enhance' in file_lower:
            region = 'enhancer'

        #print(file)
        df = pd.read_csv(file, names=['names', 'power'], sep='\t')
        #print(df)
        df = rem_methpoint(df)

        groups = df.groupby('names', sort=False)
        out_lst = []
        for i, group in groups:
            out_lst.append(group_to_row(group, region))
        out_df = pd.DataFrame(out_lst, columns=['region_start', 'region_end',
                                'meth_points_all', 'powers', 'tissue']).reset_index(drop=True)

        out_df = out_df[['region_start', 'region_end', 'meth_points_all', 'powers', 'tissue']]
        out_filename = out_folder+'/'+samplename+'_'+methylation+'_'+region+'_allMRs.csv'
        out_df.to_csv(out_filename, index=False)
        out_filenames.append(out_filename)
    return out_filenames
