import pandas as pd
import argparse
import numpy as np

def quantile_normalization(non_norm_meth_df):
    """Uses quantile normalization to normalize raw methylation file, based on the
    explanation from Wikipedia(https://en.wikipedia.org/wiki/Quantile_normalization)"""

    df_copy = non_norm_meth_df[['HpaII', 'BGT', 'MspI']].copy()
    for col in df_copy:
        # sorting each column, increasing
        df_copy[col] = np.sort(df_copy[col])

    # creating a rank for each row, based on the mean among the columns
    rank = df_copy.mean(axis=1).tolist()

    # putting the ranks in the right places of non_norm_meth_df
    for col in ['HpaII', 'BGT', 'MspI']:
        t = np.searchsorted(np.sort(non_norm_meth_df[col]), non_norm_meth_df[col])
        non_norm_meth_df[col] = [rank[i] for i in t]

    return non_norm_meth_df


def main(raw_methylation_file, gzip, out_folder):
    """Applies quantile normalization across the three libraries(HpaII, BGT, MspI)
    in raw HMST-Seq data file"""
    if gzip:
        non_norm_meth_df = pd.read_csv(raw_methylation_file, sep='\t', skiprows=[0],
                                 names=('chr', 'site', 'HpaII', 'BGT', 'MspI'),
                                 compression='gzip')

        norm_meth = quantile_normalization(non_norm_meth_df)

        samplename = raw_methylation_file.split('/')[-1][:-7]
        out_filename = out_folder+'/'+samplename+'.normalization.txt.gz'
        non_norm_meth_df.to_csv(out_filename, sep='\t', index=None, compression='gzip')
    else:
        non_norm_meth_df = pd.read_csv(raw_methylation_file, sep='\t', skiprows=[0],
                                 names=('chr', 'site', 'HpaII', 'BGT', 'MspI'))

        norm_meth = quantile_normalization(non_norm_meth_df)

        samplename = raw_methylation_file.split('/')[-1][:-4]
        out_filename = out_folder+'/'+samplename+'.normalization.txt'
        non_norm_meth_df.to_csv(out_filename, sep='\t', index=None)


    return out_filename
