import argparse
import os
import sys
from .scripts.preprocess_reference import main as pr_main
from .scripts.create_bed import main as cb_main

def main(reference_file, genome_file, human, numerical_chr_name, remove_mir, X, Y,
         M, N, intergenic_between_genes, min_intergenic_len, max_intergenic_len, remove_short, folder_out):
    """Calls preprocess_reference and create_bed, which cleans the reference and
    extracts genomic regions from the genes of the reference"""

    out_list_file = folder_out+'/list_region_files.txt'
    folder_out += '/data'

    ref_bed = pr_main(reference_file, human, numerical_chr_name, remove_mir, folder_out)

    #added wang
    region_files = cb_main(ref_bed, genome_file, X, Y, M, N, intergenic_between_genes,
                           min_intergenic_len, max_intergenic_len, remove_short, folder_out)


    outfiles = open(out_list_file, 'w')
    #added jbw
    TSS, TES, gene, dist5, intergenic, removed , dist5D = region_files
    outfiles.write(TSS+'\n')
    outfiles.write(gene+'\n')
    outfiles.write(TES+'\n')
    outfiles.write(dist5+'\n')
    outfiles.write(intergenic+'\n')
    #added jbw
    outfiles.write(dist5D +'\n')
    outfiles.close()

    return out_list_file

def my_parser(parser):
    required = parser.add_argument_group("Required")
    required.add_argument("-r", "--referenceFile",
                        metavar='file',
                        required=True,
                        help="reference file")
    required.add_argument("-g", "--genomeFile",
                        metavar='file',
                        required=True,
                        help="the genome file, one column with chromosome names and one column with the length of each chromosome")
    required.add_argument("-hu", "--human",
                        metavar='y/m/r',
                        required=True,
                        help="is sample from human? [yes/y] or [mouse/m] or [rat/r]. if no, it must be from either mouse/m or rat/r genome")
    required.add_argument("-n", "--numericalChr",
                        metavar='y/n',
                        required=True,
                        help="will you operate with numerical chromosome names or not, [yes/y] or [no/n]. the same as chosen here must be as in the genomeFile, default=no")

    optional = parser.add_argument_group("Optional, has default values")
    optional.add_argument("-rm", "--removeMir",
                        metavar='',
                        default='yes',
                        help="remove genes with names starting with 'Mir'? [yes/y] or [no/n]. default=yes")
    optional.add_argument("-X",
                        metavar='',
                        type=int,
                        default=1000,
                        help="the number of upstream bp, TSS, TES, gene. default=1000")
    optional.add_argument("-Y",
                        metavar='',
                        type=int,
                        default=1000,
                        help="the number of downstream bp, TSS, TES, gene. default=1000.")
    optional.add_argument("-M",
                        metavar='',
                        type=int,
                        default=10000,
                        help="the number of bp from gene start site, 5dist. default=10000.")
    optional.add_argument("-N",
                        metavar='',
                        type=int,
                        default=1000000,
                        help="the number of bp from gene start site, 5dist. default=1000000.")
    optional.add_argument("-i", "--intergenicBTGenes",
                        metavar='',
                        default='yes',
                        help="intergenic regions is between gene body regions [yes/y], or between TSS and TES [no/n]?. default=yes")
    optional.add_argument("-l", "--minIntergenicLen",
                        metavar='',
                        default=2000,
                        type=int,
                        help="minimum intergenic region distance. default=2000")
    #added wang
    optional.add_argument("-xL","--maxIntergenicLen",
                          metavar='',
                          default=10000000,
                          type=int,
                          help="maximum intergenic region distance, default=10000000")
    optional.add_argument("-rem", "--removeShort",
                        metavar='',
                        default='yes',
                        help="should regions be removed from TSS, TES and 5distance regions if gene body is removed for its size being less than 0? [yes/y] or [no/n]. default=yes")
    optional.add_argument("-F", "--folderOut",
                        metavar='',
                        default='out',
                        help="what should be the name of the out-folder. default=out")
    return parser

def run(args):
    """Calls the main function with the unpacked arguments"""
    if args.folderOut[-1] == '/':
        args.folderOut = args.folderOut[:-1]

    if not os.path.exists(args.folderOut):
        os.system('mkdir '+args.folderOut)

    if not os.path.exists(args.folderOut+'/data'):
        os.system('mkdir '+args.folderOut+'/data')

    #added wang
    print("5' distance regions ", args.M, args.N)
    print("minimum , maximum length of intergenic region ", args.minIntergenicLen, args.maxIntergenicLen)

    main(args.referenceFile, args.genomeFile, args.human, args.numericalChr, args.removeMir,
         args.X, args.Y, args.M, args.N, args.intergenicBTGenes, args.minIntergenicLen,args.maxIntergenicLen,
         args.removeShort, args.folderOut)

if __name__ == "__main__":
    args = my_parser(argparse.ArgumentParser('python gene_annotation.py')).parse_args()

    run(args)
